{# Remove when NEMO 7.3.0 is released #}
<span class="step-links">
    {% if page.has_previous %}
        <a href="?{{ request.META.QUERY_STRING }}&o={{ paginator.order_by }}&p=1&pp={{ paginator.per_page }}">&laquo; first</a>
        <a href="?{{ request.META.QUERY_STRING }}&o={{ paginator.order_by }}&p={{ page.previous_page_number }}&pp={{ paginator.per_page }}">previous</a>
    {% endif %}
    <span class="current">Page {{ page.number }} of {{ paginator.num_pages }}</span>
    {% if page.has_next %}
        <a href="?{{ request.META.QUERY_STRING }}&o={{ paginator.order_by }}&p={{ page.next_page_number }}&pp={{ paginator.per_page }}">next</a>
        <a href="?{{ request.META.QUERY_STRING }}&o={{ paginator.order_by }}&p={{ paginator.num_pages }}&pp={{ paginator.per_page }}">last &raquo;</a>
    {% endif %}
</span>
