{% load custom_tags_and_filters %}
{% if tickets %}
    <div class="col-lg-10 col-md-12 table-responsive" style="margin-top: 15px;float: none;padding: 0">
        {% include "NEMO_osticket/snippets/tickets_table.html" with sortable=False %}
    </div>
{% else %}
    {% if not osticket_api_available %}<div>There are no open tickets for this tool.</div>{% endif %}
{% endif %}
{% if osticket_api_available %}
    {% url 'osticket_create_tool_ticket' tool_id as create_ticket_url %}
    {% include 'NEMO_osticket/snippets/osticket_create_ticket.html' with create_ticket_url=create_ticket_url %}
{% endif %}
<div style="height:150px">{# Spacer #}</div>
