{% extends "base.html" %}
{% block content %}
    {% if form.errors %}
        <div class="alert alert-danger">
            Oops! Something went wrong. Please correct the errors highlighted below.
            <br>
            {{ form.errors }}
        </div>
    {% endif %}
    {% url "osticket_create_ticket" as create_ticket_url %}
    {% include "NEMO_osticket/snippets/osticket_create_ticket.html" with create_ticket_url=create_ticket_url %}
{% endblock %}
